package cn.ezhizao.project.business.product.controller;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import cn.ezhizao.common.utils.bean.BeanUtils;
import cn.ezhizao.common.utils.poi.ExcelUtil;
import cn.ezhizao.framework.aspectj.lang.annotation.Log;
import cn.ezhizao.framework.aspectj.lang.enums.BusinessType;
import cn.ezhizao.framework.redis.RedisCache;
import cn.ezhizao.framework.web.controller.BaseController;
import cn.ezhizao.framework.web.domain.AjaxResult;
import cn.ezhizao.framework.web.page.TableDataInfo;
import cn.ezhizao.project.business.carrierReject.domain.BizCarrierReject;
import cn.ezhizao.project.business.carrierReject.service.IBizCarrierRejectService;
import cn.ezhizao.project.business.outsource.domain.BizOutsourcedOrderDetail;
import cn.ezhizao.project.business.outsource.domain.BizOutsourcedOrderDetailProcess;
import cn.ezhizao.project.business.outsource.service.IBizOutsourcedOrderDetailProcessService;
import cn.ezhizao.project.business.outsource.service.IBizOutsourcedOrderDetailService;
import cn.ezhizao.project.business.product.domain.*;
import cn.ezhizao.project.business.product.service.*;
import cn.ezhizao.project.business.technologicalProcessDetail.domain.BizTechnologicalProcessDetail;
import cn.ezhizao.project.business.technologicalProcessDetail.service.IBizTechnologicalProcessDetailService;
import cn.ezhizao.project.system.domain.SysUser;
import cn.ezhizao.project.system.service.ISysUserService;
import cn.hutool.core.date.DateTime;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;


/**
 * 序检主Controller
 *
 * @author ezhizao
 * @date 2024-04-24
 */
@RestController
@RequestMapping("/business/inspecion")
public class BizProcessInspecionController extends BaseController {
    @Resource
    private IBizProcessInspecionService bizProcessInspecionService;

    @Resource
    private ISysUserService sysUserService;

    @Resource
    private IBizDayworkCarrierService bizDayworkCarrierService;
    @Resource
    private IBizLotTechnologicalProcessDetailService bizLotTechnologicalProcessDetailService;

    @Resource
    private IBizTechnologicalProcessDetailService bizTechnologicalProcessDetailService;

    @Resource
    private IBizDayworkItemRejectService bizDayworkItemRejectService;

    @Resource
    private IBizDayworkItemConsultService bizDayworkItemConsultService;

    @Resource
    private IBizCarrierRejectService bizCarrierRejectService;

    @Resource
    private IBizDayworkService bizDayworkService;

    @Resource
    private IBizProductService bizProductService;

    @Resource
    private IBizDayworkItemService bizDayworkItemService;

    @Resource
    private IBizLotService bizLotService;

    @Resource
    private IBizCarrierService bizCarrierService;
    @Resource
    private IBizCarrierCategoryService bizCarrierCategoryService;

    @Resource
    private IBizProductService productService;
    @Resource
    private IBizReturnReceiptDetailService bizReturnReceiptDetailService;
    @Resource
    private IBizProductionPlanDetailService productionPlanDetailService;
    @Resource
    private IBizProcessInspectionDetailService processInspectionDetailService;
    @Resource
    private IBizOutsourcedOrderDetailService bizOutsourcedOrderDetailService;
    @Resource
    private IBizOutsourcedOrderDetailProcessService bizOutsourcedOrderDetailProcessService;
    @Resource
    private IBizDayworkItemConsultPictureService bizDayworkItemConsultPictureService;
    @Resource
    private RedisCache redisCache;

    final private String processInspectionNoKey = "processInspectionCode";


    /**
     * 查询序检主列表
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @PostMapping("/list")
    public TableDataInfo list(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {

        if (bizProcessInspecion.getStartTime() != null) {
            Date startTimeDate = bizProcessInspecion.getStartTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time = dateFormat.format(startTimeDate);

            bizProcessInspecion.setStartTimeString(time + " 00:00:00");
            bizProcessInspecion.setEndTimeString(time + " 23:59:59");
        }


        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        return getDataTable(list);
    }

    /**
     * 查询序检主列表(外协序检)
     */
//    @PreAuthorize("@ss.hasPermi('business:inspecion:list')")
    @PostMapping("/outsourcedlist")
    public TableDataInfo outsourcedlist(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        if (bizProcessInspecion.getStartTime() != null) {
            Date startTimeDate = bizProcessInspecion.getStartTime();
            // 使用 SimpleDateFormat 格式化 Date 对象为特定格式的字符串
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
            String time = dateFormat.format(startTimeDate);

            bizProcessInspecion.setStartTimeString(time + " 00:00:00");
            bizProcessInspecion.setEndTimeString(time + " 23:59:59");
        }
        List<BizProcessInspecion> list = bizProcessInspecionService.getOutsourcedlist(bizProcessInspecion);
        return getDataTable(list);
    }

    /**
     * 查询产品信息
     */
    @PostMapping("/getProductConsult")
    public AjaxResult getProductConsult(@RequestBody BizLot lot) throws NoSuchFieldException, IllegalAccessException {
        BizProduct product = bizProductService.getById(lot.getProductId());
        if (product == null) {
            throw (new RuntimeException("该产品不存在"));
        }

        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        product.setTechnicianName(user == null ? "" : user.getNickName());

        return success(product);
    }

    /**
     * 查询序捡详情
     *
     * @param processInspecion
     * @return
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     */
    @PostMapping("/selectInspecion")
    public AjaxResult selectInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.query().eq("process_inspection_id", processInspecion.getId()).eq("is_process_inspection", 1).list();
        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setDayworkItemConsults(itemConsults);
        processInspecion.setDayworkItemRejects(itemRejects);

        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());

        BizLot lot = new BizLot();
        lot.setId(processInspecion.getLotId());
        lot = bizLotService.getAllList(lot).get(0);

        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }

        lot.setAllCarrierName(dayworkItem.getCarrierName());
        lot.setDayworkItemId(dayworkItem.getId());
        lot.setCarrierCode(dayworkItem.getCarrierCode());
        lot.setCarrierId(dayworkItem.getCarrierId());
        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
        processInspecion.setLot(lot);
        return success(processInspecion);
    }


    @PostMapping("/selectOutsourcedInspecion")
    public AjaxResult selectOutsourcedInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        List<BizDayworkItemConsult> itemConsults = bizDayworkItemConsultService.query().eq("process_inspection_id", processInspecion.getId()).eq("is_outsourced_inspection", 1).list();
        List<BizDayworkItemReject> itemRejects = bizDayworkItemRejectService.query().eq("process_inspection_id", processInspecion.getId()).list();
        List<BizProcessInspectionDetail> reject = processInspectionDetailService.query().eq("process_inspection_id", processInspecion.getId()).list();
        processInspecion.setDayworkItemConsults(itemConsults);
        processInspecion.setDayworkItemRejects(itemRejects);
        processInspecion.setRejects(reject);

        BizDayworkItem dayworkItem = bizDayworkItemService.selById(processInspecion.getDayworkItemId());

        BizLot lot = new BizLot();
        lot.setId(processInspecion.getLotId());
        lot = bizLotService.getAllList(lot).get(0);

        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(dayworkItem.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }

        lot.setAllCarrierName(dayworkItem.getCarrierName());
        lot.setDayworkItemId(dayworkItem.getId());
        lot.setCarrierCode(dayworkItem.getCarrierCode());
        lot.setCarrierId(dayworkItem.getCarrierId());
        lot.setPudName(dayworkItem.getProdNum() == 0 ? lot.getProductionQuantity() : dayworkItem.getProdNum());
        processInspecion.setLot(lot);
        return success(processInspecion);
    }

    /**
     * 新增外协序检信息
     */
//    @PostMapping("/saveOutsourcedInspecion")
    @Transactional
    public AjaxResult saveOutsourcedInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        System.out.println("DayworkID:" + dayworkItem.getDayworkId());
        System.out.println("getTechnologicalProcessDetailId:" + dayworkItem.getTechnologicalProcessDetailId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();
        System.out.println("回收:" + bizReturnReceiptDetail);
        //根据dayworkItem的工序ID和dayworkId查询回收Id
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }

        if (processInspecion.getStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());
        }

        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        //保存废品信息
        List<BizDayworkItemReject> rejects = processInspecion.getDayworkItemRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        if (rejects != null && rejects.size() > 0) {
            for (BizDayworkItemReject reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        bizDayworkItemRejectService.saveBatch(rejects);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item-> item.getId()==null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                    BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                    consultInfo.setStatus(consult.getStatus());
                    consultInfo.setContent(consult.getContent());
                    consultInfo.setConsultDepartment(consult.getConsultDepartment());
                    consultInfo.setUserId(user == null ? 0L : user.getUserId());
                    consultInfo.setNickName(user == null ? "" : user.getNickName());
                    consultInfo.setDayworkId(daywork.getId());
                    consultInfo.setProcessInspectionId(processInspecion.getId());
                    consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                    consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    consultInfo.setLotId(processInspecion.getLot().getId());
                    consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                    consultInfo.setDayworkId(daywork.getId());
                    consultInfo.setDayworkItemId(dayworkItem.getId());
                    consultInfo.setProductId(processInspecion.getLot().getProductId());
                    consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                    consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                    consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                    consultInfo.setProcessId(dayworkItem.getProcessId());
                    consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                    consultInfo.setIsOutsourcedInspection(1);
                    consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                    needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne ->{
                consults.forEach(item ->{
                    item.getPictures().forEach(info ->{
                        BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                        consultPicture.setDayworkItemConsultId(ne.getId());
                        consultPicture.setPictureName(info.getPictureName());
                        consultPicture.setUrl(info.getUrl());
                        pictures.add(consultPicture);
                    });
                });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }


        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem) );
    }
    @PostMapping("/getProcessInspectionByDayworkId")
    public AjaxResult getProcessInspectionByDayworkId(@RequestBody BizProcessInspecion processInspecion) {
        List<BizProcessInspectionDetail> list = bizProcessInspecionService.getInspectionByDayworkId(processInspecion);
        List<BizDayworkItemConsult> bizDayworkItemConsultList = new ArrayList<>();
        if(!list.isEmpty()) {
            //咨询信息
            List<Long> inspectionIds = list.stream().map(BizProcessInspectionDetail::getProcessInspectionId).distinct().collect(Collectors.toList());
            bizDayworkItemConsultList = bizDayworkItemConsultService.getListByInspectionIds(inspectionIds);
        }
        // 并将每个分组的元素收集到一个 List 中
        Map<Long, List<BizProcessInspectionDetail>> groupedByProcessInspectionId =
                list.stream()
                        .collect(Collectors.groupingBy(
                                BizProcessInspectionDetail::getProcessInspectionId, // 分组依据的函数
                                Collectors.toList() // 指定分组内的元素收集为 List
                        ));
        List<BizProcessInspecion> result = new ArrayList<>();
        for (Map.Entry<Long, List<BizProcessInspectionDetail>> entry : groupedByProcessInspectionId.entrySet()) {
                List<BizProcessInspectionDetail> details = entry.getValue();
                BizProcessInspecion bizProcessInspecion = new BizProcessInspecion();
                bizProcessInspecion.setProcessAlias(details.get(0).getProcessAlias());
                bizProcessInspecion.setNickName(details.get(0).getInspectionName());
                bizProcessInspecion.setRemark(details.get(0).getRemark());
                bizProcessInspecion.setStatus(details.get(0).getStatus());
                bizProcessInspecion.setId(details.get(0).getProcessInspectionId());
                List<BizProcessInspectionDetail> rejectList = new ArrayList<>();
                for (BizProcessInspectionDetail reject : details) {
                    BizProcessInspectionDetail rejectInfo = new BizProcessInspectionDetail();
                    rejectInfo.setCheckStandard(reject.getCheckStandard());
                    rejectInfo.setRejectNum(reject.getRejectNum());
                    rejectInfo.setCheckResult(reject.getCheckResult());
                    rejectList.add(rejectInfo);
                }
                bizProcessInspecion.setProcessInspectionDetails(rejectList);
            if(!bizDayworkItemConsultList.isEmpty()) {
                bizProcessInspecion.setDayworkItemConsults(bizDayworkItemConsultList.stream().filter(item -> item.getProcessInspectionId().equals(bizProcessInspecion.getId())).collect(Collectors.toList()));
            }
                result.add(bizProcessInspecion);
        }
        return success(result);
    }

    /**
     * 新增序检信息
     */
    @PostMapping("/saveProcessInspecion")
    @Transactional
    public AjaxResult saveProcessInspecion(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }

        if (processInspecion.getStatus() != 0) {
            processInspecion.setEndTime(new Date());

        }
        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        //保存废品信息
        List<BizDayworkItemReject> rejects = processInspecion.getDayworkItemRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        if (rejects != null && rejects.size() > 0) {
            for (BizDayworkItemReject reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
            }
        }
        bizDayworkItemRejectService.saveBatch(rejects);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item-> item.getId()==null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                    BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                    consultInfo.setConsultDepartment(consult.getConsultDepartment());
                    consultInfo.setUserId(user == null ? 0L : user.getUserId());
                    consultInfo.setNickName(user == null ? "" : user.getNickName());
                    consultInfo.setDayworkId(daywork.getId());
                    consultInfo.setContent(consult.getContent());
                    consultInfo.setProcessInspectionId(processInspecion.getId());
                    consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                    consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    consultInfo.setLotId(processInspecion.getLot().getId());
                    consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                    consultInfo.setDayworkId(daywork.getId());
                    consultInfo.setDayworkItemId(dayworkItem.getId());
                    consultInfo.setProductId(processInspecion.getLot().getProductId());
                    consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                    consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                    consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                    consultInfo.setProcessId(dayworkItem.getProcessId());
                    consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                    consultInfo.setIsProcessInspection(1);
                    consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                    consultInfo.setStatus(consult.getStatus());
                    consultInfo.setPictures(consult.getPictures());
                    needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne ->{
                    ne.getPictures().forEach(info ->{
                        BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                        consultPicture.setDayworkItemConsultId(ne.getId());
                        consultPicture.setPictureName(info.getPictureName());
                        consultPicture.setUrl(info.getUrl());
                        pictures.add(consultPicture);
                    });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }


        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem) );
    }

    private boolean checkOutsourceDayworkItem(Long outsourceOrderDetailId) {
        // 检测外协序检后是否已经周转到下一工序。
        BizDayworkItem bizDayworkItem = bizDayworkItemService.query().eq("outsource_detail_id", outsourceOrderDetailId).orderByDesc("create_time").last("limit 1").one();
        BizDaywork bizDaywork = bizDayworkService.getById(bizDayworkItem.getDayworkId());
        List<BizDayworkItem> items = bizDayworkItemService.query().eq("daywork_id", bizDayworkItem.getDayworkId()).list();
        // 下序已经报工
        // 下序未报工
        return items.stream().noneMatch(v -> v.getCreateTime().compareTo(bizDayworkItem.getCreateTime()) > 0 && v.getStatus().compareTo("0") >= 0 && v.getStatus().compareTo("3") <= 0);
    }

    @Transactional
//    @PostMapping("/saveOutsourcedInspecion")
    public AjaxResult saveOutsourcedInspectionV3(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
//        // 判断是否已有未确认外协检查
//        if (bizProcessInspecionService.query().eq("outsource_order_detail_id", processInspecion.getOutsourceOrderDetailId()).eq("status", 0).count() > 0) {
//            return error("已有未完成外协检查单。");
//        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());

        // 判断是否已经进行过质检
        BizProcessInspecion previousItem = bizProcessInspecionService.query().eq("outsource_order_detail_id", bizReturnReceiptDetail.getOutsourceDetailId()).one();
        if (previousItem != null) {
            if (processInspecion.getId() == null) {
                previousItem.setRecheckDate(DateTime.now().toJdkDate());
            }
            previousItem.setIsInspectionCarrierChanged(0);
            previousItem.setInspectionCarrierChangedTime(null);
            previousItem.setStatus(processInspecion.getStatus());
            previousItem.setRejectNum(processInspecion.getRejectNum());
            previousItem.setExaminingNum(processInspecion.getExaminingNum());
            previousItem.setDisqualificationNum(processInspecion.getDisqualificationNum());
        } else {
            previousItem = new BizProcessInspecion(processInspecion);
            String beginCode = "WDD" + DateTime.now().toString("yyMM");
            String previous = redisCache.hasKey(processInspectionNoKey) ? redisCache.getCacheObject(processInspectionNoKey) : "";
            String codeValue = "";
            if (previous.isEmpty()) {
                previous = (beginCode + "000001");
            } else {
                codeValue = previous;
                int lastFourDigits = Integer.parseInt(codeValue.substring(7, 13));
                lastFourDigits += 1; // 加1
                codeValue = codeValue.substring(0, 7) + String.format("%06d", lastFourDigits);
                previous = codeValue;
            }
            previousItem.setOutsourcedInspectionNo(previous);
            redisCache.setCacheObject(processInspectionNoKey, previous);
        }
        previousItem.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
        previousItem.setUserId(user == null ? 0L : user.getUserId());
        previousItem.setNickName(user == null ? "" : user.getNickName());
        previousItem.setCarrierId(processInspecion.getLot().getCarrierId());
        previousItem.setCarrierCode(processInspecion.getLot().getCarrierCode());
        previousItem.setDayworkId(daywork.getId());
        previousItem.setProductionPlanId(daywork.getProductionPlanId());
        previousItem.setDayworkItemId(dayworkItem.getId());
        previousItem.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        previousItem.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        previousItem.setProductDescription(processInspecion.getLot().getProductDescription());
        previousItem.setLotId(processInspecion.getLot().getId());
        previousItem.setLotCode(processInspecion.getLot().getLotCode());
        previousItem.setDeptId(daywork.getDeptId());
        previousItem.setProductId(processInspecion.getLot().getProductId());
        previousItem.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        previousItem.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        previousItem.setInspectionCarrierId(processInspecion.getInspectionCarrierId());

        if (previousItem.getStartTime() == null) {
            previousItem.setStartTime(new Date());
        }
        if (previousItem.getStatus() != 0) {
            previousItem.setEndTime(new Date());
            previousItem.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());

        }
        if (!checkOutsourceDayworkItem(processInspecion.getOutsourceOrderDetailId())) {
            return error("编辑失败，下序已经生产加工");
        }

        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(daywork.getId()))) {
            return error("检测载具已绑定其他批次");
        }

        //保存
        bizProcessInspecionService.saveOrUpdate(previousItem);

        if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(daywork.getId())) && (previousItem.getIsInspectionCarrierChanged() == null || previousItem.getIsInspectionCarrierChanged().equals(0))) {
            // 该检测载具需要绑定到当前批次
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(previousItem.getDayworkId());
            newDayworkCarrier.setCarrierCode(carrier.getCode());
            newDayworkCarrier.setCarrierId(carrier.getId());
            newDayworkCarrier.setLotCode(previousItem.getLotCode());
            newDayworkCarrier.setLotId(previousItem.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(previousItem.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(previousItem.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }
        List<BizDayworkItemConsult> consults = new ArrayList<>();
        if (processInspecion.getId() != null) {
            //保存废品信息
            List<BizProcessInspectionDetail> rejects = processInspecion.getRejects();
            //保存前先删除原先废品信息
            BizDayworkItemReject rej = new BizDayworkItemReject();
            rej.setProcessInspectionId(processInspecion.getId());
            bizDayworkItemRejectService.removeReject(rej);
            BizProcessInspectionDetail inspectionDetail = new BizProcessInspectionDetail();
            inspectionDetail.setProcessInspectionId(processInspecion.getId());
            processInspectionDetailService.physicalDelete(inspectionDetail);
            BizDayworkItemReject newRej = new BizDayworkItemReject();
            newRej.setRejectNum(0L);

            if (rejects != null && rejects.size() > 0) {
                for (BizProcessInspectionDetail reject : rejects) {
                    reject.setId(null);
                    reject.setUserId(user == null ? 0L : user.getUserId());
                    reject.setProcessInspectionId(processInspecion.getId());
                    reject.setDayworkId(0L);
                    reject.setProductionPlanId(daywork.getProductionPlanId());
                    reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    reject.setLotId(processInspecion.getLot().getId());
                    reject.setDayworkItemId(dayworkItem.getId());
                    Long rejectNum = reject.getRejectNum() == null ? 0L : reject.getRejectNum();
                    newRej.setRejectNum(newRej.getRejectNum());
                    newRej.setUserId(user == null ? 0L : user.getUserId());
                    newRej.setProcessInspectionId(processInspecion.getId());
                    newRej.setDayworkId(0L);
                    newRej.setProductionPlanId(daywork.getProductionPlanId());
                    newRej.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    newRej.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    newRej.setLotId(processInspecion.getLot().getId());
                    newRej.setDayworkItemId(dayworkItem.getId());
                    newRej.setReason("外协检查废品");
                    newRej.setDeptId(processInspecion.getDeptId());
                }
            }
            processInspectionDetailService.saveBatch(rejects);
            bizDayworkItemRejectService.save(newRej);
            //保存咨询信息
            consults.addAll(processInspecion.getDayworkItemConsults());
            if (consults.size() > 0) {
                for (BizDayworkItemConsult consult : consults) {
                    consult.setUserId(user == null ? 0L : user.getUserId());
                    consult.setNickName(user == null ? "" : user.getNickName());
                    consult.setDayworkId(daywork.getId());
                    consult.setProcessInspectionId(processInspecion.getId());
                    consult.setProductionPlanId(daywork.getProductionPlanId());
                    consult.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                    consult.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                    consult.setLotId(processInspecion.getLot().getId());
                    consult.setLotCode(processInspecion.getLot().getLotCode());
                    consult.setDayworkId(daywork.getId());
                    consult.setDayworkItemId(dayworkItem.getId());
                    consult.setProductId(processInspecion.getLot().getProductId());
                    consult.setProductDescription(processInspecion.getLot().getProductDescription());
                    consult.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                    consult.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                    consult.setProcessId(dayworkItem.getProcessId());
                    consult.setProcessAlias(processInspecion.getLot().getProcessAlias());
                    consult.setIsOutsourcedInspection(1);
                    consult.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                }
            }
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem) && (consults.isEmpty() || bizDayworkItemConsultService.saveOrUpdateBatch(consults)));
    }


    @Transactional
    @PostMapping("/saveOutsourcedInspecion")
    public AjaxResult saveOutsourcedInspectionV2(@RequestBody BizProcessInspecion processInspecion) throws NoSuchFieldException, IllegalAccessException {
//        // 判断是否已有未确认外协检查
//        if (bizProcessInspecionService.query().eq("outsource_order_detail_id", processInspecion.getOutsourceOrderDetailId()).eq("status", 0).count() > 0) {
//            return error("已有未完成外协检查单。");
//        }
        //根据lot信息保存序检表
        BizDaywork daywork = bizDayworkService.query().eq("lot_code", processInspecion.getLot().getLotCode()).one();
        BizProduct product = bizProductService.getById(processInspecion.getLot().getProductId());
        SysUser technicianUser = sysUserService.selectUserById(product.getTechnicianId());
        BizDayworkItem dayworkItem = bizDayworkItemService.getById(processInspecion.getLot().getDayworkItemId());
        BizReturnReceiptDetail bizReturnReceiptDetail = bizReturnReceiptDetailService.query().eq("daywork_id", dayworkItem.getDayworkId()).eq("technological_process_detail_id", dayworkItem.getTechnologicalProcessDetailId()).eq("deleted", 0).one();

        // 判断是否已经进行过质检
//        BizProcessInspecion previousItem = bizProcessInspecionService.query().eq("outsource_order_detail_id", bizReturnReceiptDetail.getOutsourceDetailId()).one();
//        previousItem.setRecheckDate(DateTime.now().toJdkDate());

        SysUser user = sysUserService.selectUserById(processInspecion.getUser().getUserId());
        processInspecion.setReturnReceiptDetailId(bizReturnReceiptDetail.getId());
        processInspecion.setUserId(user == null ? 0L : user.getUserId());
        processInspecion.setNickName(user == null ? "" : user.getNickName());
        processInspecion.setCarrierId(processInspecion.getLot().getCarrierId());
        processInspecion.setCarrierCode(processInspecion.getLot().getCarrierCode());
        processInspecion.setDayworkId(daywork.getId());
        processInspecion.setProductionPlanId(daywork.getProductionPlanId());
        processInspecion.setDayworkItemId(dayworkItem.getId());
        processInspecion.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
        processInspecion.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
        processInspecion.setProductDescription(processInspecion.getLot().getProductDescription());
        processInspecion.setLotId(processInspecion.getLot().getId());
        processInspecion.setLotCode(processInspecion.getLot().getLotCode());
        processInspecion.setDeptId(daywork.getDeptId());
        processInspecion.setProductId(processInspecion.getLot().getProductId());
        processInspecion.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
        processInspecion.setInspectionCarrierCode(processInspecion.getInspectionCarrierCode());
        processInspecion.setInspectionCarrierId(processInspecion.getInspectionCarrierId());
        if (processInspecion.getStartTime() == null) {
            processInspecion.setStartTime(new Date());
        }
        if (processInspecion.getStatus() != 0) {
            processInspecion.setEndTime(new Date());
            processInspecion.setFirstUpdaterId(processInspecion.getFirstUpdaterId() == null || processInspecion.getFirstUpdaterId().equals(0L) ? user != null ? user.getUserId() : 0L : processInspecion.getFirstUpdaterId());

        }
        if (!checkOutsourceDayworkItem(processInspecion.getOutsourceOrderDetailId())) {
            return error("编辑失败，下序已经生产加工");
        }

        BizCarrier carrier = bizCarrierService.query().eq("id", processInspecion.getInspectionCarrierId()).one();
        List<BizDayworkCarrier> dayworkCarriers = bizDayworkCarrierService.query().eq("carrier_id", carrier.getId()).eq("is_changed", 0).list();
        if (dayworkCarriers.stream().anyMatch(v -> !v.getDayworkId().equals(processInspecion.getDayworkId()))) {
            return error("检测载具已绑定其他批次");
        }

        //保存
        bizProcessInspecionService.saveOrUpdate(processInspecion);

        if (dayworkCarriers.stream().noneMatch(v -> v.getDayworkId().equals(processInspecion.getDayworkId())) && (processInspecion.getIsInspectionCarrierChanged() == null || processInspecion.getIsInspectionCarrierChanged().equals(0))) {
            // 该检测载具需要绑定到当前批次
            BizDayworkCarrier newDayworkCarrier = new BizDayworkCarrier();
            newDayworkCarrier.setDayworkId(processInspecion.getDayworkId());
            newDayworkCarrier.setCarrierCode(carrier.getCode());
            newDayworkCarrier.setCarrierId(carrier.getId());
            newDayworkCarrier.setLotCode(processInspecion.getLotCode());
            newDayworkCarrier.setLotId(processInspecion.getLotId());
            newDayworkCarrier.setProductionPlanDetailId(processInspecion.getProductionPlanDetailId());
            newDayworkCarrier.setProcessInspectionId(processInspecion.getId());
            bizDayworkCarrierService.save(newDayworkCarrier);
        }

        //保存废品信息
        List<BizProcessInspectionDetail> rejects = processInspecion.getRejects();
        //保存前先删除原先废品信息
        BizDayworkItemReject rej = new BizDayworkItemReject();
        rej.setProcessInspectionId(processInspecion.getId());
        bizDayworkItemRejectService.removeReject(rej);
        BizProcessInspectionDetail inspectionDetail = new BizProcessInspectionDetail();
        inspectionDetail.setProcessInspectionId(processInspecion.getId());
        processInspectionDetailService.physicalDelete(inspectionDetail);
        BizDayworkItemReject newRej = new BizDayworkItemReject();
        newRej.setRejectNum(0L);

        if (rejects != null && rejects.size() > 0) {
            for (BizProcessInspectionDetail reject : rejects) {
                reject.setId(null);
                reject.setUserId(user == null ? 0L : user.getUserId());
                reject.setProcessInspectionId(processInspecion.getId());
                reject.setDayworkId(0L);
                reject.setProductionPlanId(daywork.getProductionPlanId());
                reject.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                reject.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                reject.setLotId(processInspecion.getLot().getId());
                reject.setDayworkItemId(dayworkItem.getId());
                Long rejectNum = reject.getRejectNum() == null? 0L : reject.getRejectNum();
                newRej.setRejectNum(newRej.getRejectNum());
                newRej.setUserId(user == null ? 0L : user.getUserId());
                newRej.setProcessInspectionId(processInspecion.getId());
                newRej.setDayworkId(0L);
                newRej.setProductionPlanId(daywork.getProductionPlanId());
                newRej.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                newRej.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                newRej.setLotId(processInspecion.getLot().getId());
                newRej.setDayworkItemId(dayworkItem.getId());
                newRej.setReason("外协检查废品");
                newRej.setDeptId(processInspecion.getDeptId());
            }
        }
        processInspectionDetailService.saveBatch(rejects);
        bizDayworkItemRejectService.save(newRej);
        //保存咨询信息

        List<BizDayworkItemConsult> consults = processInspecion.getDayworkItemConsults().stream().filter(item-> item.getId()==null).collect(Collectors.toList());
        List<BizDayworkItemConsult> needAdd = new ArrayList<>();
        if (consults != null && consults.size() > 0) {
            for (BizDayworkItemConsult consult : consults) {
                BizDayworkItemConsult consultInfo = new BizDayworkItemConsult();
                consultInfo.setPictures(consult.getPictures());
                consultInfo.setUserId(user == null ? 0L : user.getUserId());
                consultInfo.setNickName(user == null ? "" : user.getNickName());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setContent(consult.getContent());
                consultInfo.setConsultDepartment(consult.getConsultDepartment());
                consultInfo.setProcessInspectionId(processInspecion.getId());
                consultInfo.setProductionPlanId(daywork.getProductionPlanId());
                consultInfo.setProductionPlanDetailId(daywork.getProductionPlanDetailId());
                consultInfo.setProductionPlanDetailSubDetailId(daywork.getProductionPlanDetailSubDetailId());
                consultInfo.setLotId(processInspecion.getLot().getId());
                consultInfo.setLotCode(processInspecion.getLot().getLotCode());
                consultInfo.setDayworkId(daywork.getId());
                consultInfo.setDayworkItemId(dayworkItem.getId());
                consultInfo.setProductId(processInspecion.getLot().getProductId());
                consultInfo.setProductDescription(processInspecion.getLot().getProductDescription());
                consultInfo.setTechnologicalProcessId(processInspecion.getLot().getTechnologicalProcessId());
                consultInfo.setTechnologicalProcessDetailId(dayworkItem.getTechnologicalProcessDetailId());
                consultInfo.setProcessId(dayworkItem.getProcessId());
                consultInfo.setProcessAlias(processInspecion.getLot().getProcessAlias());
                consultInfo.setIsOutsourcedInspection(1);
                consultInfo.setTechnicianId(technicianUser == null ? 0L : technicianUser.getUserId());
                needAdd.add(consultInfo);
            }
            bizDayworkItemConsultService.saveBatch(needAdd);
            List<BizDayworkItemConsultPicture> pictures = new ArrayList<>();
            needAdd.forEach(ne ->{
                ne.getPictures().forEach(info ->{
                        BizDayworkItemConsultPicture consultPicture = new BizDayworkItemConsultPicture();
                        consultPicture.setDayworkItemConsultId(ne.getId());
                        consultPicture.setPictureName(info.getPictureName());
                        consultPicture.setUrl(info.getUrl());
                        pictures.add(consultPicture);
                    });
            });
            bizDayworkItemConsultPictureService.saveBatch(pictures);
        }

        //修改daywaork信息
        dayworkItem.setIsProcessInspection(1);
        return success(bizDayworkItemService.saveOrUpdate(dayworkItem));
    }


//    /**
//     * 根据箱号查询是否为一箱多批
//     */
    //原代码备份
//    @PostMapping("/getCarrierInfo")
//    public AjaxResult getCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
//        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(dayworkCarrier);
//        if (list.size() == 0) {
//            throw new RuntimeException("该箱号未绑定任何批次");
//        }
//
//        return success(list);
//    }

    /**
     * 根据箱号查询是否为检查载具，或一箱多批
     */
//    @PostMapping("/getCarrierInfo")
    @PostMapping("/getCarrierInfoProcess")
    public AjaxResult getCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
//        List<BizDayworkCarrier> list = bizDayworkCarrierService.query().eq("carrier_code",dayworkCarrier.getCarrierCode()).eq("deleted",0).eq("is_changed",0).list();
        dayworkCarrier.setIsChanged(0);
        //先查询到箱号信息
        BizCarrier carrier = bizCarrierService.query().eq("code", dayworkCarrier.getCarrierCode()).eq("deleted", 0).one();
        if (carrier == null) {
            return error("未找到该箱号");
        }
        //查询箱号是否废弃
        BizCarrierReject carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).one();
        if (carrierReject != null) {
            return error("该载具已废弃");
        }

        //根据箱号查询绑定信息，如果是检查载具并且没有绑定信息返回载具信息，如果是检查载具且有绑定信息，返回警告
        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(dayworkCarrier);
        if (carrier.getIsInspection().equals(1) && list.size() == 0) {
            return success(carrier);
        } else if (carrier.getIsInspection().equals(1) && list.size() > 0) {
            return error("检查载具:" + carrier.getCode() + "已经绑定其他批次");
        }
        if (list.size() == 0) {
            throw new RuntimeException("该箱号未绑定任何批次");
        }
        return success(list);
    }

    @PostMapping("/getCarrierInfo")
    public AjaxResult getCarrierInfoByType(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        // 判断是哪种箱
        String carrierCode = dayworkCarrier.getCarrierCode();
        BizCarrier carrier = bizCarrierService.query().eq("code", carrierCode).one();
        if (carrier == null) {
            return error("未找到该箱号");
        }
        //查询箱号是否废弃
        BizCarrierReject carrierReject = bizCarrierRejectService.query().eq("carrier_id", carrier.getId()).eq("deleted", 0).eq("is_abandoned", 1).last("limit 1").one();
        if (carrierReject != null) {
            return error("该载具已废弃");
        }

        Map<String, Object> map = new HashMap<>();
        dayworkCarrier.setIsChanged(0);
        List<BizDayworkCarrier> list = bizDayworkCarrierService.getList(dayworkCarrier);
        List<BizDaywork> dayworks = bizDayworkService.query().in("id", list.isEmpty() ? Collections.singletonList(0L) : list.stream().map(BizDayworkCarrier::getDayworkId).collect(Collectors.toList())).list();
        if (carrier.getIsInspection().equals(1)) {
            // 为检查载具返回检查载具信息
            if (carrier.getIsInspection().equals(1) && list.size() == 0) {
                map.put("inspectionCarrier", carrier);
                return success(map);
            } else if (carrier.getIsInspection().equals(1) && list.size() > 0) {
                // 需要显示已经绑定的批次。
                return error("检查载具:" + carrier.getCode() + "已经绑定其他批次" + dayworks.stream().map(BizDaywork::getLotCode).collect(Collectors.joining()));
            }
        } else {
            if (list.size() == 0) {
                throw new RuntimeException("该箱号未绑定任何批次。");
            }
            map.put("dayworkCarriers", list);
            return success(map);
        }
        return success();
    }

    /**
     * 根据箱号查询批次信息
     */
    @PostMapping("/getLotOutsourcedInfo")
    public AjaxResult getLotOutsourcedInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        List<String> codes = new ArrayList<>();
        codes.add("27110");
        codes.add("28030");
        codes.add("28080");
        codes.add("27070");
        List<String> processCodes = dayworkCarrier.getProcessCode() == null ? codes : dayworkCarrier.getProcessCode();
        System.out.println("查询工序编码：processCodes" + processCodes);
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            throw new RuntimeException("该箱号未绑定任何批次");
        }
        System.out.println("请求信息：dayworkCarrier" + dayworkCarrier);

        //根据dayworkId查询外协回收信息是否有该批次
        BizReturnReceiptDetail bizReturnReceiptDetail = new BizReturnReceiptDetail();
        bizReturnReceiptDetail.setDayworkId(dayworkCarrier.getDayworkId());
        List<BizReturnReceiptDetail> detailList = bizReturnReceiptDetailService.getList(bizReturnReceiptDetail);
        List<BizOutsourcedOrderDetailProcess> processes = bizOutsourcedOrderDetailProcessService.query().in("detail_id", detailList.isEmpty() ? Collections.singletonList(0L) : detailList.stream().map(BizReturnReceiptDetail::getOutsourceDetailId).collect(Collectors.toList())).list();
        System.out.println("查询是否有该批次：detailList" + detailList);
        if (detailList.size() == 0) {
            throw new RuntimeException("该批次不是外协回收批次");
        }

        if (processes.stream().noneMatch(detail -> processCodes.stream().anyMatch(l -> l.trim().equals(detail.getProcessCode())))) {
            throw new RuntimeException("该批次不需要外协检查");
        }
//                        现在先不考虑 审核通过后才能外协检查限制先放到下一迭代 20240625
//        20240703 加回去
        if (detailList.stream().noneMatch(detail -> detail.getIsAudit().equals(1) && detail.getStatus().equals(1))) {
            throw new RuntimeException("该批次需要审核通过后才能外协检查");
        }
        detailList = detailList.stream()
                .filter(detail -> processes.stream().anyMatch(v -> v.getTechnologicalProcessDetailId().equals(detail.getTechnologicalProcessDetailId())))
                .filter(detail -> processCodes.stream().anyMatch(l -> l.trim().equals(detail.getProcessCode())) &&
                        detail.getIsAudit().equals(1)
//                        现在先不考虑 审核通过后才能外协检查限制先放到下一迭代 20240625
//                        20240703加回去
                        && detail.getStatus().equals(1)
                )
                .collect(Collectors.toList());
        //如果有多道工序，则根据工序的排序找到最后一条
        if (detailList.size() > 1) {
            //按照工序排序
            detailList.sort(Comparator.comparing(BizReturnReceiptDetail::getProcessStepNumber, Comparator.reverseOrder()));
        }
        //根据工序id查询报工信息
        BizDaywork daywork = new BizDaywork();
        daywork.setId(detailList.get(0).getDayworkId());
        daywork.setTechnologicalProcessDetailId(detailList.get(0).getTechnologicalProcessDetailId());

        List<BizDayworkItem> itemList = bizDayworkItemService.getByDaywork(daywork);
        if (itemList.size() == 0) {
            throw new RuntimeException("该批次未开始加工");
        }
        BizDayworkItem item = itemList.get(0);
        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            throw new RuntimeException("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }
        lot.setAllCarrierName(item.getCarrierName());
        lot.setDayworkItemId(item.getId());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        // 获取外协工序列表
        BizOutsourcedOrderDetail detail = bizOutsourcedOrderDetailService.getById(detailList.get(0).getOutsourceDetailId());
        lot.setOutsourceOrderDetailId(detail != null ? detail.getId() : null);
        lot.setProcessList(detail != null ? detail.getProcessNames() : "");
        return success(lot);
    }

    /**
     * 根据箱号查询批次信息
     */
//    @PreAuthorize("@ss.hasPermi('business:inspection:list')")
    @PostMapping("/getLotInfo")
    public AjaxResult getLotInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
        //根据箱号查询当前绑定信息
        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
        if (dayworkCarrier == null) {
            return error("该箱号未绑定任何批次");
        }

        //获取报工信息
        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId());
        if (itemList.size() == 0) {
            return error("该批次未开始加工");
        }
        BizDayworkItem item = itemList.get(0);
        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
        BizProduct product = productService.getById(productionPlanDetail.getProductId());
        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
        if (user != null) {
            item.setTechnicianName(user.getNickName());
            item.setTechnicianId(user.getUserId());
        } else {
            item.setTechnicianName("");
            item.setTechnicianId(0L);
        }

        //查询是否为正常批次
        BizLot lot = new BizLot();
        lot.setId(item.getLotId());
        lot = bizLotService.getAllList(lot).get(0);
        if (lot.getIsWaste() == 1) {
            return error("该批次已批废");
        }
        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        } else {
            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
            lot.setProcessAlias(process.getProcessAlias());
        }
        lot.setAllCarrierName(item.getCarrierName());
        lot.setDayworkItemId(item.getId());
        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
        lot.setCarrierId(dayworkCarrier.getCarrierId());
        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
        return success(lot);
    }

//    /**
//     * 根据箱号查询批次信息
//     */
//    @PostMapping("/getLotInfo")
//    public AjaxResult getLotOrCarrierInfo(@RequestBody BizDayworkCarrier dayworkCarrier) throws NoSuchFieldException, IllegalAccessException {
//
//        /** 因为需要绑定检查载具所以需要先查询出载具信息 **/
//        List<BizCarrier> carriers=bizCarrierService.query().eq("code",dayworkCarrier.getCarrierCode()).list();
//        if(carriers.isEmpty()){
//            return error("箱号信息错误，未找到该载具");
//        }
//        BizCarrier carrier=carriers.get(0);
//        //根据箱号查询当前绑定信息
//        dayworkCarrier = bizDayworkCarrierService.getByCarrierCode(dayworkCarrier);
//        /** 判断载具类型，是检查载具并且没有 **/
//        if (dayworkCarrier == null) {
//            return error("该箱号未绑定任何批次");
//        }
//
//        //获取报工信息
//        List<BizDayworkItem> itemList = bizDayworkItemService.getByDayworkId(dayworkCarrier.getDayworkId());
//        if (itemList.size() == 0) {
//            return error("改批次未开始加工");
//        }
//        BizDayworkItem item = itemList.get(0);
//        BizProductionPlanDetail productionPlanDetail = productionPlanDetailService.getById(item.getProductionPlanDetailId());
//        BizProduct product = productService.getById(productionPlanDetail.getProductId());
//        SysUser user = sysUserService.selectUserById(product.getTechnicianId());
//        if (user != null) {
//            item.setTechnicianName(user.getNickName());
//            item.setTechnicianId(user.getUserId());
//        } else {
//            item.setTechnicianName("");
//            item.setTechnicianId(0L);
//        }
//
//        //查询是否为正常批次
//        BizLot lot = new BizLot();
//        lot.setId(item.getLotId());
//        lot = bizLotService.getAllList(lot).get(0);
//        if (lot.getIsWaste() == 1) {
//            return error("改批次已批废");
//        }
//        if (lot.getIsAmend() == 1 || lot.getIsWasteRecycling() == 1) {
//            BizLotTechnologicalProcessDetail process = bizLotTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
//            lot.setProcessAlias(process.getProcessAlias());
//        } else {
//            BizTechnologicalProcessDetail process = bizTechnologicalProcessDetailService.getById(item.getTechnologicalProcessDetailId());
//            lot.setProcessAlias(process.getProcessAlias());
//        }
//        lot.setAllCarrierName(item.getCarrierName());
//        lot.setDayworkItemId(item.getId());
//        lot.setCarrierCode(dayworkCarrier.getCarrierCode());
//        lot.setCarrierId(dayworkCarrier.getCarrierId());
//        lot.setPudName(item.getProdNum() == 0 ? lot.getProductionQuantity() : item.getProdNum());
//        return success(lot);
//    }


//

    /**
     * 导出序检主列表
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:export')")
    @Log(title = "序检主", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        List<BizProcessInspecion> list = bizProcessInspecionService.getList(bizProcessInspecion);
        ExcelUtil<BizProcessInspecion> util = new ExcelUtil<BizProcessInspecion>(BizProcessInspecion.class);
        util.exportExcel(response, list, "序检主数据");
    }

    /**
     * 获取序检主详细信息
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return success(bizProcessInspecionService.getById(id));
    }

    /**
     * 新增序检主
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:add')")
    @Log(title = "序检主", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.save(bizProcessInspecion));
    }

    /**
     * 修改序检主
     */
    @PreAuthorize("@ss.hasPermi('business:inspecion:edit')")
    @Log(title = "序检主", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody BizProcessInspecion bizProcessInspecion) throws NoSuchFieldException, IllegalAccessException {
        setTenantId(bizProcessInspecion);
        return toAjax(bizProcessInspecionService.updateById(bizProcessInspecion));
    }

    /**
     * 删除序检主
     */
    @Log(title = "序检主", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @Transactional
    public AjaxResult remove(@PathVariable List<Long> ids) {
        List<BizProcessInspectionDetail> bizProcessInspectionDetails = processInspectionDetailService.query().in("process_inspection_id", ids).list();
        List<BizDayworkCarrier> carriers = bizDayworkCarrierService.query().in("process_inspection_id", ids).list();
        List<BizDayworkItemReject> rejects = bizDayworkItemRejectService.query().in("process_inspection_id", ids).list();
        List<BizDayworkItemConsult> consults = bizDayworkItemConsultService.query().in("process_inspection_id", ids).list();

        processInspectionDetailService.removeBatchByIds(bizProcessInspectionDetails);
        bizDayworkItemRejectService.removeBatchByIds(rejects);
        bizDayworkCarrierService.removeBatchByIds(carriers);
        bizDayworkItemConsultService.removeBatchByIds(consults);
        return toAjax(bizProcessInspecionService.removeBatchByIds(ids));
    }

    /**
     * 保存序检箱
     */
    @Log(title = "序检换箱", businessType = BusinessType.INSERT)
    @PostMapping("/saveInspectionCarrier")
    public AjaxResult saveInspectionCarrier(@RequestBody BizProcessInspecion bizProcessInspecion) {
        List<BizDayworkCarrier> bizDayworkCarriers = bizDayworkCarrierService.query().eq("process_inspection_id", bizProcessInspecion.getId()).eq("is_changed", 0).list();
        BizProcessInspecion current = bizProcessInspecionService.getById(bizProcessInspecion.getId());
        List<BizDayworkCarrier> newCarriers = bizProcessInspecion.getDayworkCarriers();
        if (bizProcessInspecion.getStatus().equals(0) && newCarriers.isEmpty()) {
            return error("该序检未完成，不能解绑载具");
        }
        List<BizCarrierReject> carrierRejects = bizProcessInspecion.getRejectCarrier();
        List<BizCarrier> carriers = bizCarrierService.query().in("id", carrierRejects.isEmpty() ?
                Collections.singletonList(0L) :
                carrierRejects.stream().map(BizCarrierReject::getCarrierId)
                        .collect(Collectors.toList())).list();
        carriers.forEach(l -> l.setIsAbandoned(1));
        bizCarrierService.updateBatchById(carriers);
        bizCarrierRejectService.saveBatch(carrierRejects);
        bizDayworkCarriers.forEach(l -> l.setIsChanged(1));
        bizDayworkCarrierService.updateBatchById(bizDayworkCarriers);
        if (newCarriers.isEmpty()) {
            current.setIsInspectionCarrierChanged(1);
            current.setInspectionCarrierChangedTime(new Date());
            bizProcessInspecionService.updateById(current);
        } else {
            current.setIsInspectionCarrierChanged(0);
            current.setInspectionCarrierChangedTime(null);
            current.setInspectionCarrierId(newCarriers.get(0).getCarrierId());
            current.setInspectionCarrierCode(newCarriers.get(0).getCarrierCode());
            bizProcessInspecionService.updateById(current);
        }
        newCarriers.forEach(l -> {
            l.setId(null);
            l.setIsChanged(0);
            l.setProcessInspectionId(bizProcessInspecion.getId());
            l.setDayworkId(bizProcessInspecion.getDayworkId());
            l.setProductionPlanId(bizProcessInspecion.getProductionPlanId());
            l.setLotId(bizProcessInspecion.getLotId());
        });
        bizDayworkCarrierService.saveOrUpdateBatch(newCarriers);
        return success();
    }

}
